﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MarVSM
{
    class Program
    {
        static void Main(string[] args)
        {
            var points = from x in Enumerable.Range(1, 25)
                         from y in Enumerable.Range(1, 25)
                         select new { x, y };

            foreach (var item in points)
                Console.WriteLine(item);
        }

        internal sealed class AnonymousOne
        {
            // Fields
            private readonly int xField;
            private readonly int yField;

            // Methods
            public AnonymousOne(int x, int y)
            {
                this.xField = x;
                this.yField = y;
            }

            public override bool Equals(object value)
            {
                var type = value as AnonymousOne;
                return (((type != null) &&
                    EqualityComparer<int>.Default.Equals(this.xField, type.xField)) &&
                    EqualityComparer<int>.Default.Equals(this.yField, type.yField));
            }

            public override int GetHashCode()
            {
                uint num = 0xdd9cb371;
                num = (0xa5555529 * num) + 
                    (uint)EqualityComparer<int>.Default.GetHashCode(this.xField);
                return (int)((0xa5555529 * num) + 
                    (uint)EqualityComparer<int>.Default.GetHashCode(this.yField));
            }

            public override string ToString()
            {
                StringBuilder builder = new StringBuilder();
                builder.Append("{ x = ");
                builder.Append(this.xField);
                builder.Append(", y = ");
                builder.Append(this.yField);
                builder.Append(" }");
                return builder.ToString();
            }

            // Properties
            public int x
            {
                get { return this.xField; }
            }

            public int y
            {
                get { return this.yField; }
            }
        }

        internal sealed class HandCodedPoint : 
            IEquatable<HandCodedPoint>, 
            IComparable<HandCodedPoint>,
            IComparable
        {
            // Fields
            private readonly int xField;
            private readonly int yField;

            // Methods
            public HandCodedPoint(int x, int y)
            {
                this.xField = x;
                this.yField = y;
            }

            #region IEquatable<AnonymousTwo> Members

            public bool Equals(HandCodedPoint other)
            {
                return (((other != null) &&
                    EqualityComparer<int>.Default.Equals(this.xField, other.xField)) &&
                    EqualityComparer<int>.Default.Equals(this.yField, other.yField));
            }

            #endregion
            public override bool Equals(object value)
            {
                return Equals(value as HandCodedPoint);
            }

            public override int GetHashCode()
            {
                uint num = 0xdd9cb371;
                num = (0xa5555529 * num) +
                    (uint)EqualityComparer<int>.Default.GetHashCode(this.xField);
                return (int)((0xa5555529 * num) +
                    (uint)EqualityComparer<int>.Default.GetHashCode(this.yField));
            }

            public static bool operator ==(HandCodedPoint left, HandCodedPoint right)
            {
                if (left == null)
                    return right == null;
                return left.Equals(right);
            }
            public static bool operator !=(HandCodedPoint left, HandCodedPoint right)
            {
                return !(left == right);
            }

            public override string ToString()
            {
                StringBuilder builder = new StringBuilder();
                builder.Append("{ x = ");
                builder.Append(this.xField);
                builder.Append(", y = ");
                builder.Append(this.yField);
                builder.Append(" }");
                return builder.ToString();
            }

            // Properties
            public int x
            {
                get { return this.xField; }
            }

            public int y
            {
                get { return this.yField; }
            }

            #region IComparable<HandCodedPoint> Members

            public int CompareTo(HandCodedPoint other)
            {
                // Something is greater than nothing:
                if (other == null)
                    return 1;
                int distanceSquared = xField * xField + yField * yField;
                int otherDistanceSquared = other.xField * other.xField + 
                    other.yField + other.yField;
                return distanceSquared.CompareTo(otherDistanceSquared);

            }

            #endregion

            #region IComparable Members

            public int CompareTo(object obj)
            {
                return CompareTo(obj as HandCodedPoint);
            }

            #endregion

            public static bool operator >(HandCodedPoint left, HandCodedPoint right)
            {
                if (left == null)
                    return false;
                else
                    return left.CompareTo(right) > 0;
            }
            public static bool operator <(HandCodedPoint left, HandCodedPoint right)
            {
                if (left == null)
                    return right != null;
                else
                    return left.CompareTo(right) < 0;
            }
            public static bool operator >=(HandCodedPoint left, HandCodedPoint right)
            {
                return (left == right) || (left > right);
            }
            public static bool operator <=(HandCodedPoint left, HandCodedPoint right)
            {
                return (left == right) || (left < right);
            }
        }

    }
}
